@@ -29,9 +29,28 @@
 
 #include "xdgdirs.h"
 
-#include <QtTest>
+#include <QDir>
+#include <QFileInfo>
+#include <QRandomGenerator>
+#include <QTest>
 #include <QTemporaryDir>
 
+using namespace Qt::Literals::StringLiterals;
+
+QString randomString(int length)
+{
+    static const QString characterPool("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"_L1);
+    if (length < 0)
+        return QString();
+
+    QString randomString;
+    for (int i = 0; i < length; ++i) {
+        const int index = QRandomGenerator::global()->generate() % characterPool.size();
+        randomString.append(characterPool.at(index));
+    }
+    return randomString;
+}
+
 class tst_xdgdirs : public QObject
 {
     Q_OBJECT
@@ -47,10 +66,15 @@ private Q_SLOTS:
     void testCacheHome();
     void testAutostartHome();
     void testAutostartDirs();
+    void testNonWritableLocations();
 
 private:
     void setDefaultLocations();
     void setCustomLocations();
+    void setNonWritableLocations();
+
+    QString noSlashPostfix;
+    QString postfix;
 
     QString m_configHome;
     QTemporaryDir m_configHomeTemp;
@@ -62,18 +86,28 @@ private:
     QTemporaryDir m_dataDirsTemp;
     QString m_cacheHome;
     QTemporaryDir m_cacheHomeTemp;
+
+    QDir m_nonWritableRoot;
+    QString m_configHomeNonWritable;
 };
 
 void tst_xdgdirs::initTestCase()
 {
     QCoreApplication::instance()->setOrganizationName(QStringLiteral("QtXdg"));
     QCoreApplication::instance()->setApplicationName(QStringLiteral("tst_xdgdirs"));
+
+    noSlashPostfix = QCoreApplication::applicationName();
+    postfix = u'/' + QCoreApplication::applicationName();
 }
 
 void tst_xdgdirs::cleanupTestCase()
 {
     QCoreApplication::instance()->setOrganizationName(QString());
     QCoreApplication::instance()->setApplicationName(QString());
+
+    QFile f(m_nonWritableRoot.path());
+    QVERIFY(f.setPermissions(QFile::WriteUser | QFile::WriteOwner));
+    QVERIFY(m_nonWritableRoot.removeRecursively());
 }
 
 void tst_xdgdirs::setDefaultLocations()
@@ -100,12 +134,27 @@ void tst_xdgdirs::setCustomLocations()
 
 }
 
+void tst_xdgdirs::setNonWritableLocations()
+{
+    const QString randomDir = randomString(8);
+    m_nonWritableRoot = QDir::temp();
+
+    QVERIFY(m_nonWritableRoot.mkdir(randomDir, QFileDevice::ReadUser));
+    m_nonWritableRoot.cd(randomDir);
+    m_configHomeNonWritable = m_nonWritableRoot.path() + u'/' + "nonwritabledir"_L1;
+
+    qputenv("XDG_CONFIG_HOME", QFile::encodeName(m_configHomeNonWritable));
+    qputenv("XDG_DATA_HOME", QFile::encodeName(m_configHomeNonWritable));
+    qputenv("XDG_CACHE_HOME", QFile::encodeName(m_configHomeNonWritable));
+}
+
 void tst_xdgdirs::testDataHome()
 {
     setDefaultLocations();
     const QString expectedDataHome = QDir::homePath() + QString::fromLatin1("/.local/share");
     QCOMPARE(XdgDirs::dataHome(), expectedDataHome);
     QCOMPARE(XdgDirs::dataHome(false), expectedDataHome);
+    QCOMPARE(XdgDirs::dataHome(true), expectedDataHome);
 
     setCustomLocations();
     QCOMPARE(XdgDirs::dataHome(), m_dataHome);
@@ -126,8 +175,6 @@ void tst_xdgdirs::testConfigHome()
 
 void tst_xdgdirs::testDataDirs()
 {
-    const QString postfix = QString::fromLatin1("/") + QCoreApplication::applicationName();
-
     setDefaultLocations();
     const QStringList dataDirs = XdgDirs::dataDirs();
     QCOMPARE(dataDirs.count(), 2);
@@ -139,6 +186,11 @@ void tst_xdgdirs::testDataDirs()
     QCOMPARE(dataDirsWithPostfix.at(0), QString::fromLatin1("/usr/local/share") + postfix);
     QCOMPARE(dataDirsWithPostfix.at(1), QString::fromLatin1("/usr/share") + postfix);
 
+    const QStringList dataDirsWithNoSlashPostfix = XdgDirs::dataDirs(noSlashPostfix);
+    QCOMPARE(dataDirsWithNoSlashPostfix.count(), 2);
+    QCOMPARE(dataDirsWithNoSlashPostfix.at(0), "/usr/local/share"_L1 + u'/' + noSlashPostfix);
+    QCOMPARE(dataDirsWithNoSlashPostfix.at(1), "/usr/share"_L1 + u'/' + noSlashPostfix);
+
     setCustomLocations();
     const QStringList dataDirsCustom = XdgDirs::dataDirs();
     QCOMPARE(dataDirsCustom.count(), 1);
@@ -147,11 +199,14 @@ void tst_xdgdirs::testDataDirs()
     const QStringList dataDirsCustomWithPostfix = XdgDirs::dataDirs(postfix);
     QCOMPARE(dataDirsCustomWithPostfix.count(), 1);
     QCOMPARE(dataDirsCustomWithPostfix.at(0), m_dataDirs + postfix);
+
+    const QStringList dataDirsCustomWithNoSlashPostfix;
+    QCOMPARE(dataDirsCustomWithPostfix.count(), 1);
+    QCOMPARE(dataDirsCustomWithPostfix.at(0), m_dataDirs + u'/' + noSlashPostfix);
 }
 
 void tst_xdgdirs::testConfigDirs()
 {
-    const QString postfix = QString::fromLatin1("/") + QCoreApplication::applicationName();
     setDefaultLocations();
 
     const QStringList configDirs = XdgDirs::configDirs();
@@ -201,8 +256,6 @@ void tst_xdgdirs::testAutostartHome()
 
 void tst_xdgdirs::testAutostartDirs()
 {
-    const QString postfix = QString::fromLatin1("/") + QCoreApplication::applicationName();
-
     setDefaultLocations();
     const QStringList autostartDirs = XdgDirs::autostartDirs();
     QCOMPARE(autostartDirs.count(), 1);
@@ -212,6 +265,9 @@ void tst_xdgdirs::testAutostartDirs()
     QCOMPARE(autostartDirsWithPostfix.count(), 1);
     QCOMPARE(autostartDirsWithPostfix.at(0), QString::fromLatin1("/etc/xdg/autostart") + postfix);
 
+    const QStringList autostartDirsWithNoSlashPostfix = XdgDirs::autostartDirs(noSlashPostfix);
+    QCOMPARE(autostartDirsWithNoSlashPostfix.count(), 1);
+    QCOMPARE(autostartDirsWithNoSlashPostfix.at(0), "/etc/xdg/autostart"_L1 + u'/' + noSlashPostfix);
 
     setCustomLocations();
     const QStringList autostartDirsCustom = XdgDirs::autostartDirs();
@@ -221,6 +277,19 @@ void tst_xdgdirs::testAutostartDirs()
     const QStringList autostartDirsCustomWithPostfix = XdgDirs::autostartDirs(postfix);
     QCOMPARE(autostartDirsCustomWithPostfix.count(), 1);
     QCOMPARE(autostartDirsCustomWithPostfix.at(0), m_configDirs + QString::fromLatin1("/autostart") + postfix);
+
+    const QStringList autostartDirsCustomWithNoSlashPostfix = XdgDirs::autostartDirs(noSlashPostfix);
+    QCOMPARE(autostartDirsCustomWithNoSlashPostfix.count(), 1);
+    QCOMPARE(autostartDirsCustomWithNoSlashPostfix.at(0), m_configDirs + "/autostart"_L1 + u'/' + noSlashPostfix);
+}
+
+void tst_xdgdirs::testNonWritableLocations()
+{
+    setNonWritableLocations();
+    QCOMPARE(XdgDirs::configHome(true), QString());
+    QCOMPARE(XdgDirs::dataHome(true), QString());
+    QCOMPARE(XdgDirs::cacheHome(true), QString());
+    QCOMPARE(XdgDirs::autostartHome(true), QString());
 }
 
 QTEST_APPLESS_MAIN(tst_xdgdirs)
